var
  _ = require('underscore'),
  Router = require('director').Router;

function defineRouteHandler(Router){
  /**
   * RouteHandler
   */
  function RouteHandler(routeDelegates){
    this._router = new Router();
    var actions = {};
    var self = this;
    _.each(routeDelegates, function(routeDelegate){
      _.chain(routeDelegate)
        .functions()
        .filter(function(x){
          return x.charAt(0) === '/';
        })
        .each(function(route){
          var action = function(){
            var args = Array.prototype.slice.call(arguments);
            routeDelegate[route].apply(routeDelegate, args);
          };
          actions[route] = action;
          self._router.on(route, action);
        });

      _.chain(routeDelegate)
        .keys()
        .filter(function(x){
          return x.charAt(0) === '/' && typeof(routeDelegate[x]) === 'string';
        })
        .each(function(route){
          var actionName = routeDelegate[route];
          self._router.on(route, actions[actionName]);
        });
    });
  }

  RouteHandler.prototype.initRoute = function(route){
    this._router.init(route);
  };
  RouteHandler.prototype.go = function(url){
    this._router.setRoute(url);
  };

  return RouteHandler;
}


exports.RouteHandler = defineRouteHandler(Router);
exports.defineRouteHandler = defineRouteHandler;